.\" Hey Emacs! This file is -*- nroff -*- source.
.\"
.\" Copyright (c) 1992 Drew Eckhardt (drew@cs.colorado.edu), March 28, 1992
.\"
.\" Permission is granted to make and distribute verbatim copies of this
.\" manual provided the copyright notice and this permission notice are
.\" preserved on all copies.
.\"
.\" Permission is granted to copy and distribute modified versions of this
.\" manual under the conditions for verbatim copying, provided that the
.\" entire resulting derived work is distributed under the terms of a
.\" permission notice identical to this one
.\" 
.\" Since the Linux kernel and libraries are constantly changing, this
.\" manual page may be incorrect or out-of-date.  The author(s) assume no
.\" responsibility for errors or omissions, or for damages resulting from
.\" the use of the information contained herein.  The author(s) may not
.\" have taken the same level of care in the production of this manual,
.\" which is licensed free of charge, as they might when working
.\" professionally.
.\" 
.\" Formatted or processed versions of this manual, if unaccompanied by
.\" the source, must acknowledge the copyright and authors of this work.
.\"
.\" Modified by Michael Haardt (u31b3hs@pool.informatik.rwth-aachen.de)
.\" Modified Wed Jul 21 22:47:01 1993 by Rik Faith (faith@cs.unc.edu)
.\" Modified 21 Aug 1994 by Michael Chastain (mec@shell.portal.com):
.\"   Fixed typoes.
.\" 
.\" Traduction 9/10/1996 Christophe BLAESS (ccb@club-internet.fr)
.\" mj 10/12/1997 (LDP man-pages 1.18)
.\" mj 26/06/2000 (LDP 1.30)
.\" mj 30/05/2001 (LDP 1.36)
.\" mj 15/01/02 LDP-man-pages 1.47
.\" mj 18/07/03 LDP-man-pages 1.56
.TH EXECVE 2 "18 juillet 2003" LDP "Manuel du programmeur Linux"
.SH NOM
execve \- Excuter un programme.
.SH SYNOPSIS
.B #include <unistd.h>
.sp
.BI "int execve (const char *" fichier ", char * const " argv
.BI "[], char * const " envp []);
.SH DESCRIPTION
\fBexecve()\fP excute le programme correspondant au \fIfichier\fP.
Celui-ci doit tre un excutable binaire ou bien un script
commenant par une ligne du type "\fB#! \fIinterprteur \fR[arg]".
Dans ce dernier cas, l'interprteur doit tre indiqu par un nom
complet, avec son chemin d'accs, et qui sera invoqu sous la forme
\fBinterprteur\fP [arg] \fIfichier\fP.

.I argv
est un tableau de chanes d'arguments passes au nouveau programme.
.I envp
est un tableau de chanes, ayant par convention la forme 
.BR "cle=valeur" ,
qui sont passes au nouveau programme comme environnement.
.I argv
ainsi que
.I envp
doivent se terminer par un pointeur NULL. Les arguments et l'environnement
sont accessibles par le nouveau programme dans sa fonction principale,
lorsqu'elle est dfinie comme \fBint main (int argc, char * argv [], char * envp [])\fR.

En cas de russite, \fBexecve()\fP ne revient pas  l'appelant, et les
segments de texte, de donnes, ainsi que la pile du processus appelant sont
remplacs par ceux du programme charg.
Le programme invoqu hrite du PID du processus appelant, et de tous
les descripteurs de fichiers qui ne possdent pas le drapeau
Close-on-Exec. Les signaux en attente destins au processus parent sont
effacs. Les signaux prts  tre intercept par le processus
appelant reprennent leur comportement par dfaut.
Le signal SIGCHLD (lorsqu'il est configur sur SIG_IGN) peut ou non tre
rinitialis  SIG_DFL.

Si l'on effectuait un 
.IR ptrace (2)
sur le programme appelant, un signal
\fBSIGTRAP\fP est envoy aprs la russite de \fBexecve()\fP.

Si le bit Set-UID est positionn sur le \fIfichier\fP du programme, l'UID
effectif du processus appelant est modifi pour prendre celui du propritaire
du fichier. De mme, lorsque le bit Set-GID est positionn, le GID effectif
est modifi pour correspondre  celui du groupe du fichier.

Si l'excutable est un fichier binaire a.out li dynamiquement, et contenant
des appels aux bibliothques partages, le linker dynamique de Linux
.BR ld<?php include ("../(1).php"); ?>est appel avant l'excution, afin de charger les bibliothques partages
ncessaires en mmoire, et d'effectuer l'dition des liens de l'excutable.

Si l'excutable est au format ELF li dynamiquement, l'interprteur
indiqu dans le segment PT_INTERP sera invoqu pour charger les bibliothques
partages. Cet interprteur est gnralement
\fI/lib/ld-linux.so.1\fR pour les fichiers binaires lis avec la libc Linux
version 5, ou \fI/lib/ld-linux.so.2\fR pour ceux lis avec la
GNU libc version 2.
.SH "VALEUR RENVOYE"
En cas de russite, \fBexecve()\fP ne revient pas, en cas d'chec il
renvoie \-1 et
.I errno
contient le code d'erreur.
.SH ERREURS
.TP
.B EACCES
Le \fIfichier\fP n'est pas un fichier rgulier.
.TP
.B EACCES
L'autorisation d'excution est refuse pour le \fIfichier\fP, ou un script, ou un interprteur ELF.
.TP
.B EACCES
Le systme de fichiers est mont avec l'option
.IR noexec .
.TP
.B EPERM
Le systme de fichiers est mont avec l'attribut 
.I nosuid
et le fichier a un bit Set\-UID ou Set\-GID positionn.
.TP
.B EPERM
Le processus est suivi avec \fBptrace\fP(2), l'utilisateur n'est pas
le Super\-User, et le fichier a un bit Set-UID ou Set-GID activ.
.TP
.B E2BIG
La liste d'arguments est trop grande.
.TP
.B ENOEXEC
Le fichier excutable n'est pas dans le bon format, ou est destin
 une autre architecture.
.TP
.B EFAULT
L'argument
.I fichier
pointe en dehors de l'espace d'adressage accessible.
.TP
.B ENAMETOOLONG
La chane de caractres
.I fichier
est trop longue.
.TP
.B ENOENT
Le 
.I fichier
ou un script ou un interprteur ELF n'existe pas, ou une bibliothque partage
ncessaire pour le fichier ou l'interprteur n'est pas disponible.
.TP
.B ENOMEM
Pas assez de mmoire pour le noyau.
.TP
.B ENOTDIR
Un lment du chemin d'accs n'est pas un rpertoire.
.TP
.B EACCES
L'accs  un lment du chemin n'est pas autoris.
.TP
.B ELOOP
Le chemin d'accs au
.I fichier 
contient une rfrence circulaire ( travers un lien symbolique)
.TP
.B ETXTBSY
Le \fIfichier\fP excutable a t ouvert en criture par un ou plusieurs
processus.
.TP
.B EIO
Une erreur d'entre/sortie de bas-niveau s'est produite.
.TP
.B ENFILE
Le nombre maximal de fichiers ouverts sur le systme est atteint
.TP
.B EMFILE
Le nombre maximal de fichiers ouverts par processus est atteint.
.TP
.B EINVAL
Un excutable ELF a plusieurs segments PT_INTERP (indique plusieurs
interprteurs).
.TP
.B EISDIR
L'interprteur ELF cit est un rpertoire.
.TP
.B ELIBBAD
L'interprteur ELF mentionn n'est pas dans un format connu.
.SH "CONFORMIT"
SVr4, SVID, X/OPEN, BSD 4.3.  POSIX ne documente pas le comportement
avec #! mais est nanmoins compatible. SVr4 dcrit des erreurs supplmentaires
EAGAIN, EINTR, ELIBACC, ENOLINK, EMULTIHOP; POSIX ne dcrit pas les conditions
d'erreur ETXTBSY, EPERM, EFAULT, ELOOP, EIO, ENFILE, EMFILE, EINVAL, EISDIR, ni
ELIBBAD.
.SH NOTES
Les processus Set\-UID et Set\-GID ne peuvent pas tre suivis par
\fBptrace()\fP.

Linux ignore les bits Set-UID et Set-GID sur les scripts.

Le rsultat d'un montage de systme de fichiers avec l'attribut
.I nosuid
peut varier suivant les versions de noyau\ : certaines refuseront l'excution
des fichiers SUID/SGID lorsque cela donnerait  l'appelant des privilges
qu'il n'a pas (et renverront l'erreur EPERM), d'autres ignoreront simplement
les bits SUID/SGID mais accepteront d'effectuer l'appel execve.

La premire ligne d'un shell script excutable (#!) a une longueur
maximale de 127 caractres.

.SH "VOIR AUSSI"
.BR chmod (2),
.BR fork (2),
.BR execl (3),
.BR environ (5),
.BR ld<?php include ("../(8)..php"); ?>.SH TRADUCTION
Christophe Blaess, 1996-2003.
