.\" -*- nroff -*-
.\" Copyright 1995 Yggdrasil Computing, Incorporated.
.\" written by Adam J. Richter (adam@yggdrasil.com),
.\" with typesetting help from Daniel Quinlan (quinlan@yggdrasil.com).
.\"
.\" This is free documentation; you can redistribute it and/or
.\" modify it under the terms of the GNU General Public License as
.\" published by the Free Software Foundation; either version 2 of
.\" the License, or (at your option) any later version.
.\"
.\" The GNU General Public License's references to "object code"
.\" and "executables" are to be interpreted as the output of any
.\" document formatting or typesetting system, including
.\" intermediate and printed output.
.\"
.\" This manual is distributed in the hope that it will be useful,
.\" but WITHOUT ANY WARRANTY; without even the implied warranty of
.\" MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
.\" GNU General Public License for more details.
.\"
.\" You should have received a copy of the GNU General Public
.\" License along with this manual; if not, write to the Free
.\" Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
.\" USA.
.\"
.\" Traduction 30/08/2000 par Christophe Blaess (ccb@club-internet.fr)
.\" LDP 1.30
.\" Mise  jour 04/06/2001 - LDP-man-pages-1.36
.\" Mise  jour 25/01/2002 - LDP-man-pages-1.47
.\" MJ 21/07/2003 LDP-1.57
.TH DLOPEN 3 "21 juillet 2003" LDP "Manuel du programmeur Linux"
.SH NOM
dlclose, dlerror, dlopen, dlsym \- Interface de programmation pour le chargeur de bibliothques dynamiques.
.SH SYNOPSIS
.B #include <dlfcn.h>
.sp
.BI "void * dlopen (const char * " filename ", int " flag );
.br
.BI "const char * dlerror(void);"
.br
.BI "void * dlsym (void * " handle ", char * " symbol );
.br
.BI "int dlclose (void * " handle );
.sp
Symboles spciaux :
.BR "_init" ", " "_fini" .
.SH DESCRIPTION
.B dlopen
charge une bibliothque dynamique depuis le fichier dont le nom est fourni dans la chane
.I filename
(termine par un caractre nul) et renvoie un descripteur opaque 
.IR "" "(" handle ")"
reprsentant la bibliothque dynamique.
Si le nom
.I filename
n'est pas un chemin absolu (commenant pas un "/"), le fichier est recherch aux endroits suivants :
.RS
.PP
La liste des rpertoires (spars par des deux-points) contenue dans la variable d'environnement \fBLD_LIBRARY_PATH\fP.
.PP
La liste des bibliothques mmorises dans \fI/etc/ld.so.cache\fP.
.PP
\fI/lib\fP, puis \fI/usr/lib\fP.
.RE
.PP
Si l'argument
.I filename
est un pointeur NULL, le descripteur renvoy correspond au programme principal.
.PP
Les rfrences externes de la bibliothque sont rsolues en utilisant les bibliothqujes
mentionnes dans sa liste de dpendances, et toutes les autres bibliothques ventuellement
ouvertes auparavant avec l'attribut
.BR RTLD_GLOBAL .
Si l'dition des liens de l'excutable a t faite avec l'option "-rdynamic", alors
ses symboles globaux seront galement employs pour rsoudre les rfrences de
la bibliothque charge dynamiquement.
.PP
L'attribut
.I flag
doit tre soit
.BR RTLD_LAZY ,
indiquant que la rsolution des symboles aura lieu lorsque le code sera excut, soit
.BR RTLD_NOW ,
demandant la rsolution de tous les symboles indfinis avant le retour de
.BR dlopen 
et l'chec de cette routine si c'est impossible.
En option, on peut ajouter (avec un OU binaire) l'attribut
.B RTLD_GLOBAL
rendant les symboles externes de la bibliothque disponible pour les bibliothques
charges ultrieurement.
.PP
Si la bibliothque exporte une routine nomme
.BR _init ,
alors ce code sera excut avant le retour de
.BR dlopen() .
Au cas o vous voudriez viter le lien avec les fichiers de dmarrage du systme,
vous pouvez prciser le paramtre "-nostartfiles" sur la ligne de commande
de gcc.
.PP
Si la mme bibliothque est charge deux fois avec
.BR dlopen() ,
le mme descripteur sera renvoy. Un compte du nombre de chargements est toutefois
conserv afin d'viter de la dcharger avant que la fonction
.B dlclose()
n'ait t appele autant de fois que
.B dlopen()
a russi.
.PP
Si
.B dlopen()
choue pour une raison quelconque, elle renvoie NULL.
Une chane de caractres en clair, dcrivant la dernire erreur rencontre dans une
routine dlopen(), dlsym() ou dlclose() peut tre rcupre en appelant
.BR dlerror() .
La fonction
.B dlerror()
renvoie NULL si aucune erreur ne s'est produite depuis l'initialisation ou depuis
sa dernire invocation (si on appelle deux fois de suite
.BR dlerror (),
le second appel donnera toujours NULL).

La fonction
.B dlsym()
prend un descripteur de bibliothque dynamique renvoye par
.BR dlopen ()
et un nom de symbole termin par un caractre nul, et renvoie l'adresse o ce symbole a t charg.
Si le symbole n'est pas trouv, 
.B dlsym()
renvoie NULL. Toutefois la bonne manire de vrifier si une erreur s'est produite dans
.B dlsym()
est d'examiner le rsultat de
.B dlerror()
(en le sauvegardant dans une variable). En effet, le symbole peut avoir lgitimement la valeur NULL
sans qu'une erreur ne se soit produite.
Le fait de sauvegarder le rsultat de
.B dlerror()
dans une variable pour examiner s'il est NULL est indispensable, car un second appel pour afficher
l'erreur renverrait toujours NULL.
.PP
Il y a deux pseudo-handles spciaux
.B RTLD_DEFAULT
et
.BR RTLD_NEXT .
le premier recherche la premire occurrence du symbole dsir en utilisant
l'ordre de recherche des bibliothques par dfaut. Le second, qui ne peut
servir que pour une bibliothque dynamique, recherchera l'occurrence suivante,
d'une fonction  partir de la bibliothque en cours.
Ceci permet de fournir un encadrement pour une fonction se trouvant
dans une autre bibliothque partage.
.PP
La fonction
.B dlclose()
dcrmente le nombre de rfrences sur la bibliothque dynamique dont le descripteur est
.IR handle .
Si ce nombre descend  zro et si aucune autre bibliothque n'emploie des symboles exports
par celle-ci, elle est dcharge. Si la bibliothque exporte une routine nomme
.BR _fini ,
alors elle est appele juste avant le dchargement.
.SH "VALEUR RENVOYE"
.B dlclose
renvoie zro si elle russi ou une valeur non-nulle en cas d'erreur.
.SH EXEMPLE
.B Charger la bibliothque mathmatique et afficher le cosinus de 2.0\ :
.RS
.nf
.if t .ft CW
#include <stdio.h>
#include <dlfcn.h>
 
int main(int argc, char **argv) {
    void *handle;
    double (*cosine)(double);
    char *error;
 
    handle = dlopen ("/lib/libm.so", RTLD_LAZY);
    if (!handle) {
        fputs (dlerror(), stderr);
        exit(1);
    }
 
    cosine = dlsym(handle, "cos");
    if ((error = dlerror()) != NULL)  {
        fprintf (stderr, "%s\en", error);
        exit(1);
    }
 
    printf ("%f\en", (*cosine)(2.0));
    dlclose(handle);
} 
.if t .ft P
.fi
.RE
.PP
Supposons que le programme s'appelle "foo.c", on doit le compiler ainsi\ :
.RS
.LP
gcc -rdynamic -o foo foo.c -ldl
.RE
.PP
Une bibliothque (disons bar.c) qui exporte _init() et _fini() sera compile
comme suit\ :
.RS
.LP
gcc -shared -nostartfiles -o bar bar.c
.RE
.SH NOTES
Les symboles RTLD_DEFAULT et RTLD_NEXT sont dfinis dans
.I <dlfcn.h>
seulement si _GNU_SOURCE a t dfinie avant l'inclusion.
.SH HISTORIQUE
L'interface standard dlopen provient de SunOS.
.SH VOIR AUSSI
.BR ld (1),
.BR ld<?php include ("../(8),.php"); ?>.BR ldconfig (8),
.BR ldd (1),
.B ld.so.info
.SH TRADUCTION
Christophe Blaess, 2000-2003.
